'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _atRule = require('postcss/lib/at-rule');

var _atRule2 = _interopRequireDefault(_atRule);

var _comment = require('postcss/lib/comment');

var _comment2 = _interopRequireDefault(_comment);

var _parser = require('postcss/lib/parser');

var _parser2 = _interopRequireDefault(_parser);

var _isMixinName = require('./tokenizer/is-mixin-name');

var _isMixinName2 = _interopRequireDefault(_isMixinName);

var _lessTokenize = require('./less-tokenize');

var _lessTokenize2 = _interopRequireDefault(_lessTokenize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var mixinRuleToken = 'mixinRule';
var mixinParamToken = 'mixin-param';
var atRuleToken = 'atrule';
var defaultOptions = {
    mixinsAsAtRules: false,
    innerMixinsAsRules: false
};

var LessParser = function (_Parser) {
    _inherits(LessParser, _Parser);

    function LessParser(input, opts) {
        _classCallCheck(this, LessParser);

        var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(LessParser).call(this, input));

        Object.assign(_this, defaultOptions, opts);
        return _this;
    }

    _createClass(LessParser, [{
        key: 'tokenize',
        value: function tokenize() {
            this.tokens = (0, _lessTokenize2.default)(this.input);
        }
    }, {
        key: 'comment',
        value: function comment(token) {
            if (token[6] === 'inline') {
                var node = new _comment2.default();

                this.init(node, token[2], token[3]);
                node.raws.inline = true;
                node.source.end = { line: token[4], column: token[5] };

                var text = token[1].slice(2);

                if (/^\s*$/.test(text)) {
                    node.text = '';
                    node.raws.left = text;
                    node.raws.right = '';
                } else {
                    var match = text.match(/^(\s*)([^]*[^\s])(\s*)$/);

                    node.text = match[2];
                    node.raws.left = match[1];
                    node.raws.right = match[3];
                }
            } else {
                _get(Object.getPrototypeOf(LessParser.prototype), 'comment', this).call(this, token);
            }
        }
    }, {
        key: 'rule',
        value: function rule(tokens) {
            var _this2 = this;

            _get(Object.getPrototypeOf(LessParser.prototype), 'rule', this).call(this, tokens);

            var selector = this.current.selector;

            /**
             * We should separate rules and mixins. According to LESS spec (http://lesscss.org/features/#mixins-feature)
             * mixins can have the same syntax as inner blocks, so we will handle only mixins as functions,
             * that contain symbol '('.
             */

            if ((0, _isMixinName2.default)(selector) && selector.indexOf('(') > 0) {
                var _ret = function () {
                    // this.current is the 'rule' node created in super.rule()
                    _this2.current.name = tokens[0][1].slice(1);

                    // parse inner mixin rule nodes as base rule node
                    if (_this2.innerMixinsAsRules && _this2.current.parent.type !== 'root') {
                        return {
                            v: void 0
                        };
                    }

                    if (_this2.mixinsAsAtRules === true) {
                        _this2.current.type = atRuleToken;
                        _this2.current.lessType = mixinRuleToken;
                    } else {
                        _this2.current.type = mixinRuleToken;
                    }

                    _this2.current.definition = true;
                    _this2.current.params = [];

                    var blockStart = tokens.findIndex(function (x) {
                        return x[0] === '(';
                    });
                    var blockEnd = tokens.findIndex(function (x) {
                        return x[0] === ')';
                    });

                    tokens.forEach(function (token) {
                        if (token[0] !== mixinParamToken) {
                            return;
                        }

                        var index = tokens.indexOf(token);
                        var param = new _atRule2.default();
                        var pos = blockStart;
                        var prev = _this2.current.params[_this2.current.params.length - 1];
                        var termToken = null;

                        param.name = token[1].slice(1);
                        param.type = mixinParamToken;
                        param.source = {
                            start: { line: token[2], column: token[3] },
                            input: _this2.input
                        };
                        param.raws = { before: '', after: '', between: '' };

                        if (token[6] && token[6] === 'rest-variables') {
                            param.restVariables = true;
                        }

                        while (pos < blockEnd) {
                            termToken = tokens[pos];

                            if (termToken[0] === ';') {
                                param.source.end = { line: termToken[2], column: termToken[3] };
                                param.raws.semicolon = true;
                                blockStart = pos + 1;
                                break;
                            } else if (termToken[0] === 'space') {
                                param.raws.before += termToken[1];

                                if (prev) {
                                    prev.raws.after += termToken[1];
                                }
                            }

                            pos += 1;
                        }

                        // this is the last param in the block
                        if (pos === blockEnd) {
                            pos = index;
                            while (pos < blockEnd) {
                                termToken = tokens[pos];

                                if (termToken[0] === 'space') {
                                    param.raws.after += termToken[1];
                                }

                                pos += 1;
                            }

                            param.source.end = { line: token[4], column: token[5] };
                        }

                        _this2.current.params.push(param);
                    });
                }();

                if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
            }
        }
    }]);

    return LessParser;
}(_parser2.default);

exports.default = LessParser;
module.exports = exports['default'];